import boto3
import re
import requests
import json


#Initialize AWS Clients for use.
CLIENT_LAMBDA = boto3.client('lambda')
CLIENT_DB = boto3.client('dynamodb')

'''
Used to raise exception when required parameters
are not included in event
'''
class InvalidParameterException(Exception):

    def __init__(self, message):
        super(message)

def lambda_handler(event, context):
    print("EVENT")
    print(event)
    
    if 'token' not in event:
        raise InvalidParameterException('You need an token to invoke this function')
    if 'deviceId' not in event:
        raise InvalidParameterException('You need an deviceId to invoke this function')

    #Construct url for forward Geocoding        
    if "forward_geocoding" in event:
        url = get_forward_url(event)
    #Construct url for reverse Geocoding 
    elif "reverse_geocoding" in event:
        url = get_reverse_url(event)
    else:
        raise InvalidParameterException('Please specify forward or reverse geocoding')

    #Invoke api request and construct payload to update device
    payload = hit_api_and_get_payload(url, event)

    print("PAYLOAD")
    print(payload)

    CLIENT_LAMBDA.invoke(
        FunctionName='fndy-put-device-secure-dev',
        InvocationType='RequestResponse',
        Payload=json.dumps(payload)
    )


'''
Parameters: 
  - url (previously constructed forward or reverse geocoding url)
  - event (Used for deviceId and constructing returned payload)
Use:
  Invoke Unwired Geocoding API with Search query if forward geocoding and 
  Long and Lat if using reverse geocoding.
Return Val:
  Payload for updating device address information.
'''
def hit_api_and_get_payload(url, event):
    print("URL")
    print(url)

    response = json.loads(requests.get(url).text)

    #Check if address info is API returned Payload
    if 'address' not in response:
        payload = {
            'body': {
                'deviceId': event['deviceId'],
                "a.display_name": "No matching address Found"
            },
            'invoke': True
        }
    else:
        #Get Address from payload
        response = response['address']

        payload = {
            'body': {
                'deviceId': event['deviceId'],
                "a.lat": "",
                "a.long": "",
                "a.display_name": "",
                "a.road": "",
                "a.county" : "",
                "a.city": "",
                "a.state": "",
                "a.country": "",
                "a.country_code": "",
                "a.postal_code": ""
            },
            'invoke': True
        }

        '''
        Forward Geocoding can return a list so returned payload will 
        contain an array represented by a string Ex state: "[WA,CA,NY]"
        '''
        if "forward_geocoding" in event:
            payload['body']['a.lat'] = "["
            payload['body']['a.long'] = "["
            payload['body']['a.display_name'] = "["
            payload['body']['a.road'] = "["
            payload['body']['a.country'] = "["
            payload['body']['a.county'] = "["
            payload['body']['a.city'] = "["
            payload['body']['a.country_code'] = "["
            payload['body']['a.postal_code'] = "["
            for address in response:
                payload = load_address_into_payload(payload, address)
                payload = add_commas(payload)
            if len(response) > 0:    
                payload = close_payload_arrays(payload)
        #Reverse Geocoding returns single address    
        elif "reverse_geocoding" in event:
            payload = load_address_into_payload(payload, response)
            payload['body']['a.lat'] = event['lat']
            payload['body']['a.long'] = event['lon']

    return payload

'''
Used to close array represented by String. Removes last commas and adds closing brace.
'''
def close_payload_arrays(payload):
    payload['body']['a.lat'] = payload['body']['a.lat'][:-1] + "]" if len(payload['body']['a.lat']) > 1 else payload['body']['a.lat']+ "]"
    payload['body']['a.long'] = payload['body']['a.long'][:-1] + "]" if len(payload['body']['a.long']) > 1 else payload['body']['a.long']+ "]"
    payload['body']['a.display_name'] = payload['body']['a.display_name'][:-1] + "]" if len(payload['body']['a.display_name']) > 1 else payload['body']['a.display_name']+ "]"
    payload['body']['a.road'] = payload['body']['a.road'][:-1] + "]" if len(payload['body']['a.road']) > 1 else payload['body']['a.road']+ "]"
    payload['body']['a.country'] = payload['body']['a.country'][:-1] + "]" if len(payload['body']['a.country']) > 1 else payload['body']['a.country']+ "]"
    payload['body']['a.county'] = payload['body']['a.county'][:-1] + "]" if len(payload['body']['a.county']) > 1 else payload['body']['a.county']+ "]"
    payload['body']['a.city'] = payload['body']['a.city'][:-1] + "]" if len(payload['body']['a.city']) > 1 else payload['body']['a.city']+ "]"
    payload['body']['a.country_code'] = payload['body']['a.country_code'][:-1] + "]" if len(payload['body']['a.country_code']) > 1 else payload['body']['a.country_code']+ "]"
    payload['body']['a.postal_code'] = payload['body']['a.postal_code'][:-1] + "]" if len(payload['body']['a.postal_code']) > 1 else payload['body']['a.postal_code']+ "]"
    return payload

'''
Add commas in between values for payload.
'''
def add_commas(payload):
    payload['body']['a.lat'] = payload['body']['a.lat'] + ","
    payload['body']['a.long'] = payload['body']['a.long'] + ","
    payload['body']['a.display_name'] = payload['body']['a.display_name'] + ","
    payload['body']['a.road'] = payload['body']['a.road'] + ","
    payload['body']['a.country'] = payload['body']['a.country'] + ","
    payload['body']['a.county'] = payload['body']['a.county'] + ","
    payload['body']['a.city'] = payload['body']['a.city'] + ","
    payload['body']['a.country_code'] = payload['body']['a.country_code'] + ","
    payload['body']['a.postal_code'] = payload['body']['a.postal_code'] + ","

    return payload

'''
If value is present in address add it to Payload. Other add "NA"
'''
def load_address_into_payload(payload, address):
    payload['body']['a.lat'] = payload['body']['a.lat'] + address['lat'] if 'lat' in address else payload['body']['a.lat'] + 'NA'
    payload['body']['a.long'] = payload['body']['a.long'] + address['lon'] if 'lon' in address else payload['body']['a.long'] + 'NA'
    payload['body']['a.display_name'] = payload['body']['a.display_name'] + address['display_name'] if 'display_name' in address else payload['body']['a.display_name'] + 'NA'
    payload['body']['a.road'] = payload['body']['a.road'] + address['road'] if 'road' in address else payload['body']['a.road'] + 'NA'
    payload['body']['a.country'] = payload['body']['a.country'] + address['country'] if 'country' in address else payload['body']['a.country'] + 'NA'
    payload['body']['a.county'] = payload['body']['a.county'] + address['county'] if 'county' in address else payload['body']['a.county'] + 'NA'
    payload['body']['a.city'] = payload['body']['a.city'] + address['city'] if 'city' in address else payload['body']['a.city'] + 'NA'
    payload['body']['a.country_code'] = payload['body']['a.country_code'] + address['country_code'] if 'country_code' in address else payload['body']['a.country_code'] + 'NA'
    payload['body']['a.postal_code'] = payload['body']['a.postal_code'] + str(address['postcode']) if 'postcode' in address else payload['body']['a.postal_code'] + 'NA'

    return payload


'''
Parameters:
  - Event (Contains arugments needed to construct API request url)
'''
def get_forward_url(event):
    '''
    Search query is the only required field. The match function is used 
    to detect if Foundry Payload is missing
    '''
    if 'q' not in event and not re.match('\[.*\]', event["q"]):
        raise InvalidParameterException('You need an Search Query to invoke this function')

    url = "https://us1.unwiredlabs.com/v2/search.php?token=" + event['token'] + "&q=" + event['q']

    if 'limit' in event:
        url = url + "&limit=" + str(event['limit'])
    if 'countrycodes' in event and not re.match('\[.*\]', event["countrycodes"]):
        url = url + "&countrycodes=" + event['countrycodes']


    return url

'''
Parameters:
  - Event (Contains arugments needed to construct API request url)
'''
def get_reverse_url(event):
    '''
    Long and Lat are required fields.
    '''
    if 'lon' not in event:
        raise InvalidParameterException('You need Longitude to invoke this function')
    if 'lat' not in event:
        raise InvalidParameterException('You need Latitude to invoke this function')

    url = ("https://us1.unwiredlabs.com/v2/reverse.php?token=" + event['token'] + "&lon=" + event['lon'] + "&lat=" + event['lat'])

    return url
